import typing as T
from dataclasses import dataclass
from enum import Enum
import graphql
from pydantic import BaseModel
from pydantic.fields import FieldInfo


class OperationType(str, Enum):
    query = "query"
    mutation = "mutation"


@dataclass
class Argument:
    name: str
    display_name: str
    value: T.Any | None


@dataclass
class Node:
    original_node: graphql.Node
    children: list[T.Union["FieldNode", "InlineFragmentNode"]] | None


@dataclass
class FieldNode(Node):
    name: str
    alias: str | None
    display_name: str
    arguments: list[Argument]
    annotation: T.Any


@dataclass
class FieldNodeField(FieldNode):
    field: FieldInfo


@dataclass
class FieldNodeMethod(FieldNode):
    method: T.Callable


@dataclass
class FieldNodeModel(FieldNode):
    models: list[T.Type[BaseModel]]


@dataclass
class InlineFragmentNode(Node):
    type_condition: str
    annotation: T.Any


@dataclass
class OperationNode(Node):
    name: str | None
    type: OperationType


__all__ = [
    "OperationType",
    "Argument",
    "Node",
    "FieldNode",
    "FieldNodeField",
    "FieldNodeModel",
    "FieldNodeMethod",
    "InlineFragmentNode",
    "OperationNode",
]



from .gql_models import GQL, GQLInput, GQLError, GQLConfigDict
from .schema_builder import SchemaBuilder
from .info import Info
from .depends import Depends
from .query_builders.edgedb.logic import get_qb
from .query_builders.edgedb.query_builder import QueryBuilder, ChildEdge
from .query_builders.edgedb.config import Link, Property, QueryBuilderConfig
from .gql_ast.models import (
    Node,
    FieldNode,
    FieldNodeModel,
    FieldNodeField,
    FieldNodeMethod,
    OperationNode,
)

build_router = SchemaBuilder.build_router

__all__ = [
    "SchemaBuilder",
    "build_router",
    "GQL",
    "GQLInput",
    "GQLError",
    "GQLConfigDict",
    "Info",
    "Depends",
    "get_qb",
    "QueryBuilder",
    "ChildEdge",
    "Link",
    "Property",
    "QueryBuilderConfig",
    "Node",
    "FieldNode",
    "FieldNodeModel",
    "FieldNodeField",
    "FieldNodeMethod",
    "OperationNode",
]



import typing as T
import dataclasses
from fastapi import Request, Response, BackgroundTasks
from fastgql.gql_ast import models as M


@dataclasses.dataclass
class Info:
    """needed to make this a raw dataclass because context needs to be kept as a reference... pydantic copies dicts"""

    variables: dict[str, T.Any] | None

    node: M.FieldNode | M.InlineFragmentNode
    parent_node: M.FieldNode | M.InlineFragmentNode | M.OperationNode
    context: dict[str, T.Any]

    request: Request
    response: Response
    background_tasks: BackgroundTasks

    errors: list[Exception]

    path: tuple[str, ...]
